;;; -*- Mode: Lisp; Package: STELLA; Syntax: COMMON-LISP; Base: 10 -*-

;;; collections.lisp

#|--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2003      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------|#

(CL:IN-PACKAGE "STELLA")

;;; Auxiliary variables:

(CL:DEFVAR SGT-COLLECTIONS-STELLA-COLLECTION NULL)
(CL:DEFVAR SYM-COLLECTIONS-STELLA-VECTOR NULL)
(CL:DEFVAR KWD-COLLECTIONS-CPP NULL)
(CL:DEFVAR KWD-COLLECTIONS-FUNCTION NULL)
(CL:DEFVAR SGT-COLLECTIONS-STELLA-VECTOR NULL)
(CL:DEFVAR SGT-COLLECTIONS-STELLA-STELLA-HASH-TABLE NULL)
(CL:DEFVAR SYM-COLLECTIONS-STELLA-SIZE NULL)
(CL:DEFVAR SYM-COLLECTIONS-STELLA-INITIAL-SIZE NULL)
(CL:DEFVAR SYM-COLLECTIONS-STELLA-FREE-ELEMENTS NULL)
(CL:DEFVAR SYM-COLLECTIONS-STELLA-STARTUP-COLLECTIONS NULL)
(CL:DEFVAR SYM-COLLECTIONS-STELLA-METHOD-STARTUP-CLASSNAME NULL)

;;; Forward declarations:

(CL:DECLAIM (CL:SPECIAL *STELLA-MODULE* *MODULE* NULL-INTEGER NIL TRUE FALSE))

;;; (DEFMETHOD (MEMBER? BOOLEAN) ...)

(CL:DEFMETHOD MEMBER? ((SELF COLLECTION) OBJECT)
  "Return true iff `object' is a member of the collection `self'."
  (CL:LET* ((I NULL) (ITER-000 (ALLOCATE-ITERATOR SELF)))
   (CL:LOOP WHILE (NEXT? ITER-000) DO (CL:SETQ I (%VALUE ITER-000))
    (CL:WHEN (EQL? I OBJECT) (CL:RETURN-FROM MEMBER? TRUE))))
  (CL:RETURN-FROM MEMBER? FALSE))

;;; (DEFUN (STELLA-COLLECTION? BOOLEAN) ...)

(CL:DEFUN STELLA-COLLECTION? (SELF)
  "Return `true' if `self' is a native collection."
  (CL:RETURN-FROM STELLA-COLLECTION?
   (ISA? SELF SGT-COLLECTIONS-STELLA-COLLECTION)))

;;; (DEFMETHOD (NO-DUPLICATES? BOOLEAN) ...)

(CL:DEFMETHOD NO-DUPLICATES? ((SELF COLLECTION))
  "Return `true' if the collection `self' forbids duplicate values."
  (CL:RETURN-FROM NO-DUPLICATES? FALSE))

;;; (DEFMETHOD (NO-DUPLICATES? BOOLEAN) ...)

(CL:DEFMETHOD NO-DUPLICATES? ((SELF SET))
  (CL:RETURN-FROM NO-DUPLICATES? TRUE))

;;; (DEFMETHOD (ORDERED? BOOLEAN) ...)

(CL:DEFMETHOD ORDERED? ((SELF COLLECTION))
  "Return `true' if the collection `self' is ordered."
  (CL:RETURN-FROM ORDERED? FALSE))

;;; (DEFMETHOD (ORDERED? BOOLEAN) ...)

(CL:DEFMETHOD ORDERED? ((SELF SEQUENCE))
  (CL:RETURN-FROM ORDERED? TRUE))

;;; (DEFMETHOD (EMPTY? BOOLEAN) ...)

(CL:DEFMETHOD EMPTY? ((SELF SET))
  "Return `true' if the set `self' has no members."
  (CL:RETURN-FROM EMPTY? (CL:EQ (%THE-CONS-LIST SELF) NIL)))

;;; (DEFMETHOD (NON-EMPTY? BOOLEAN) ...)

(CL:DEFMETHOD NON-EMPTY? ((SELF SET))
  "Return `true' if the set `self' has at least one member."
  (CL:RETURN-FROM NON-EMPTY? (CL:NOT (CL:EQ (%THE-CONS-LIST SELF) NIL))))

;;; (DEFUN (TERMINATE-SET? BOOLEAN) ...)

(CL:DEFUN TERMINATE-SET? (SELF)
  (CL:LET* ((CONS (%THE-CONS-LIST SELF)) (REST NULL))
   (CL:LOOP WHILE (CL:NOT (CL:EQ CONS NIL)) DO (CL:SETQ REST (%%REST CONS))
    (FREE CONS) (CL:SETQ CONS REST))
   (CL:SETF (%THE-CONS-LIST SELF) NIL) (CL:RETURN-FROM TERMINATE-SET? TRUE)))

;;; (DEFMETHOD (MEMBER? BOOLEAN) ...)

(CL:DEFMETHOD MEMBER? ((SELF SET) OBJECT)
  "Return `true' iff `object' is a member of the set `self'.
Uses an `eql?' test."
  (CL:LET* ((I NULL) (ITER-000 (%THE-CONS-LIST SELF)))
   (CL:LOOP WHILE (CL:NOT (CL:EQ ITER-000 NIL)) DO
    (CL:PROGN (CL:SETQ I (%%VALUE ITER-000))
     (CL:SETQ ITER-000 (%%REST ITER-000)))
    (CL:WHEN (EQL? I OBJECT) (CL:RETURN-FROM MEMBER? TRUE))))
  (CL:RETURN-FROM MEMBER? FALSE))

;;; (DEFMETHOD (LENGTH INTEGER) ...)

(CL:DEFMETHOD LENGTH ((SELF SET))
  (CL:RETURN-FROM LENGTH (LENGTH (%THE-CONS-LIST SELF))))

;;; (DEFMETHOD INSERT ...)

(CL:DEFMETHOD INSERT ((SELF SET) VALUE)
  "Add `value' to the set `self'.  First checks for duplicate."
  (CL:WHEN (CL:NOT (MEMBER? SELF VALUE))
   (CL:SETF (%THE-CONS-LIST SELF) (CONS VALUE (%THE-CONS-LIST SELF))))
  :VOID)

;;; (DEFMETHOD (REMOVE SET) ...)

(CL:DEFMETHOD REMOVE ((SELF SET) VALUE)
  "Remove all entries in `self' that match `value'."
  (CL:SETF (%THE-CONS-LIST SELF) (REMOVE (%THE-CONS-LIST SELF) VALUE))
  (CL:RETURN-FROM REMOVE SELF))

;;; (DEFMETHOD (REVERSE SET) ...)

(CL:DEFMETHOD REVERSE ((SELF SET))
  (CL:SETF (%THE-CONS-LIST SELF) (REVERSE (%THE-CONS-LIST SELF)))
  (CL:RETURN-FROM REVERSE SELF))

;;; (DEFMETHOD (COPY (SET OF (LIKE (ANY-VALUE SELF)))) ...)

(CL:DEFMETHOD COPY ((SELF SET))
  "Return a copy of the set `self'.  The conses in the copy are
freshly allocated."
  (CL:LET* ((COPY (CREATE-OBJECT (PRIMARY-TYPE SELF))))
   (CL:SETF (%THE-CONS-LIST COPY) (COPY-CONS-LIST (%THE-CONS-LIST SELF)))
   (CL:RETURN-FROM COPY COPY)))

;;; (DEFMETHOD CLEAR ...)

(CL:DEFMETHOD CLEAR ((SELF SET))
  "Make `self' an empty set."
  (CL:SETF (%THE-CONS-LIST SELF) NIL)
  :VOID)

;;; (DEFMETHOD (ALLOCATE-ITERATOR (SET-ITERATOR OF (LIKE (ANY-VALUE SELF)))) ...)

(CL:DEFMETHOD ALLOCATE-ITERATOR ((SELF SET))
  (CL:LET* ((ITERATOR (NEW-SET-ITERATOR)))
   (CL:SETF (%SET-ITERATOR-CURSOR ITERATOR) (%THE-CONS-LIST SELF))
   (CL:SETF (%SET-ITERATOR-COLLECTION ITERATOR) SELF)
   (CL:SETF (%FIRST-ITERATION? ITERATOR) TRUE)
   (CL:RETURN-FROM ALLOCATE-ITERATOR ITERATOR)))

;;; (DEFUN (TERMINATE-SET-ITERATOR? BOOLEAN) ...)

(CL:DEFUN TERMINATE-SET-ITERATOR? (SELF)
  (CL:SETF (%SET-ITERATOR-CURSOR SELF) NIL)
  (CL:SETF (%SET-ITERATOR-COLLECTION SELF) NULL)
  (CL:RETURN-FROM TERMINATE-SET-ITERATOR? TRUE))

;;; (DEFMETHOD (NEXT? BOOLEAN) ...)

(CL:DEFMETHOD NEXT? ((SELF SET-ITERATOR))
  (CL:IF (%FIRST-ITERATION? SELF)
   (CL:PROGN (CL:SETF (%FIRST-ITERATION? SELF) FALSE)
    (CL:SETF (%SET-ITERATOR-CURSOR SELF)
     (%THE-CONS-LIST (%SET-ITERATOR-COLLECTION SELF))))
   (CL:SETF (%SET-ITERATOR-CURSOR SELF) (%%REST (%SET-ITERATOR-CURSOR SELF))))
  (CL:SETF (%VALUE SELF) (%%VALUE (%SET-ITERATOR-CURSOR SELF)))
  (CL:RETURN-FROM NEXT? (CL:NOT (CL:EQ (%SET-ITERATOR-CURSOR SELF) NIL))))

;;; (DEFMETHOD (VALUE-SETTER OBJECT) ...)

(CL:DEFMETHOD VALUE-SETTER ((SELF SET-ITERATOR) VALUE)
  (CL:SETF (%%VALUE (%SET-ITERATOR-CURSOR SELF)) VALUE)
  (CL:SETF (%VALUE SELF) VALUE)
  (CL:RETURN-FROM VALUE-SETTER VALUE))

;;; (DEFMETHOD PRINT-VECTOR ...)

(CL:DEFMETHOD PRINT-VECTOR ((SELF VECTOR) STREAM)
  (CL:IF (CL:= (LENGTH SELF) 0) (%%PRINT-STREAM STREAM "|i|[]")
   (CL:LET* ((I 0) (LIMIT 9)) (CL:DECLARE (CL:TYPE CL:FIXNUM I LIMIT))
    (%%PRINT-STREAM STREAM "|i|[")
    (CL:LET*
     ((ELEMENT NULL) (VECTOR-000 SELF) (INDEX-000 0)
      (LENGTH-000 (LENGTH VECTOR-000)))
     (CL:DECLARE (CL:TYPE CL:FIXNUM INDEX-000 LENGTH-000))
     (CL:LOOP WHILE (CL:< INDEX-000 LENGTH-000) DO
      (CL:PROGN
       (CL:SETQ ELEMENT
        (CL:AREF (CL:THE CL:SIMPLE-VECTOR (%THE-ARRAY VECTOR-000)) INDEX-000))
       (CL:SETQ INDEX-000 (CL:1+ INDEX-000)))
      (%%PRINT-STREAM STREAM ELEMENT) (CL:SETQ I (CL:1+ I))
      (CL:WHEN (CL:> I LIMIT) (CL:RETURN))
      (CL:WHEN (CL:< I (LENGTH SELF)) (%%PRINT-STREAM STREAM " "))))
    (CL:IF (CL:OR (CL:<= I LIMIT) (CL:= I (LENGTH SELF)))
     (%%PRINT-STREAM STREAM "]") (%%PRINT-STREAM STREAM " ...]"))))
  :VOID)

;;; (DEFMETHOD (EMPTY? BOOLEAN) ...)

(CL:DEFMETHOD EMPTY? ((SELF VECTOR))
  "Return `true' if `self' has length 0."
  (CL:RETURN-FROM EMPTY? (CL:= (%ARRAY-SIZE SELF) 0)))

;;; (DEFMETHOD (NON-EMPTY? BOOLEAN) ...)

(CL:DEFMETHOD NON-EMPTY? ((SELF VECTOR))
  "Return `true' if `self' has length > 0."
  (CL:RETURN-FROM NON-EMPTY? (> (%ARRAY-SIZE SELF) 0)))

;;; (DEFMETHOD (FIRST (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFMETHOD FIRST ((SELF VECTOR))
  (CL:RETURN-FROM FIRST
   (CL:AREF (CL:THE CL:SIMPLE-VECTOR (%THE-ARRAY SELF)) 0)))

;;; (DEFMETHOD (SECOND (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFMETHOD SECOND ((SELF VECTOR))
  (CL:RETURN-FROM SECOND
   (CL:AREF (CL:THE CL:SIMPLE-VECTOR (%THE-ARRAY SELF)) 1)))

;;; (DEFMETHOD (THIRD (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFMETHOD THIRD ((SELF VECTOR))
  (CL:RETURN-FROM THIRD
   (CL:AREF (CL:THE CL:SIMPLE-VECTOR (%THE-ARRAY SELF)) 2)))

;;; (DEFMETHOD (FOURTH (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFMETHOD FOURTH ((SELF VECTOR))
  (CL:RETURN-FROM FOURTH
   (CL:AREF (CL:THE CL:SIMPLE-VECTOR (%THE-ARRAY SELF)) 3)))

;;; (DEFMETHOD (FIFTH (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFMETHOD FIFTH ((SELF VECTOR))
  (CL:RETURN-FROM FIFTH
   (CL:AREF (CL:THE CL:SIMPLE-VECTOR (%THE-ARRAY SELF)) 4)))

;;; (DEFMETHOD (NTH (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFMETHOD NTH ((SELF VECTOR) POSITION)
  (CL:DECLARE (CL:TYPE CL:FIXNUM POSITION))
  #+MCL
  (CL:CHECK-TYPE POSITION CL:FIXNUM)
  (CL:RETURN-FROM NTH
   (CL:AREF (CL:THE CL:SIMPLE-VECTOR (%THE-ARRAY SELF)) POSITION)))

;;; (DEFMETHOD (LAST (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFMETHOD LAST ((SELF VECTOR))
  "Return the last item in the vector `self'."
  (CL:RETURN-FROM LAST
   (CL:AREF (CL:THE CL:SIMPLE-VECTOR (%THE-ARRAY SELF))
    (CL:THE CL:FIXNUM (CL:1- (%ARRAY-SIZE SELF))))))

;;; (DEFMETHOD (FIRST-SETTER (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFMETHOD FIRST-SETTER ((SELF VECTOR) VALUE)
  (CL:RETURN-FROM FIRST-SETTER
   (CL:LET ((SELF (%THE-ARRAY SELF)) (VALUE VALUE) (POSITION 0))
    (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF) (CL:TYPE CL:FIXNUM POSITION))
    (CL:SETF (CL:AREF SELF POSITION) VALUE))))

;;; (DEFMETHOD (SECOND-SETTER (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFMETHOD SECOND-SETTER ((SELF VECTOR) VALUE)
  (CL:RETURN-FROM SECOND-SETTER
   (CL:LET ((SELF (%THE-ARRAY SELF)) (VALUE VALUE) (POSITION 1))
    (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF) (CL:TYPE CL:FIXNUM POSITION))
    (CL:SETF (CL:AREF SELF POSITION) VALUE))))

;;; (DEFMETHOD (THIRD-SETTER (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFMETHOD THIRD-SETTER ((SELF VECTOR) VALUE)
  (CL:RETURN-FROM THIRD-SETTER
   (CL:LET ((SELF (%THE-ARRAY SELF)) (VALUE VALUE) (POSITION 2))
    (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF) (CL:TYPE CL:FIXNUM POSITION))
    (CL:SETF (CL:AREF SELF POSITION) VALUE))))

;;; (DEFMETHOD (FOURTH-SETTER (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFMETHOD FOURTH-SETTER ((SELF VECTOR) VALUE)
  (CL:RETURN-FROM FOURTH-SETTER
   (CL:LET ((SELF (%THE-ARRAY SELF)) (VALUE VALUE) (POSITION 3))
    (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF) (CL:TYPE CL:FIXNUM POSITION))
    (CL:SETF (CL:AREF SELF POSITION) VALUE))))

;;; (DEFMETHOD (FIFTH-SETTER (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFMETHOD FIFTH-SETTER ((SELF VECTOR) VALUE)
  (CL:RETURN-FROM FIFTH-SETTER
   (CL:LET ((SELF (%THE-ARRAY SELF)) (VALUE VALUE) (POSITION 4))
    (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF) (CL:TYPE CL:FIXNUM POSITION))
    (CL:SETF (CL:AREF SELF POSITION) VALUE))))

;;; (DEFMETHOD (NTH-SETTER (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFMETHOD NTH-SETTER ((SELF VECTOR) VALUE POSITION)
  (CL:DECLARE (CL:TYPE CL:FIXNUM POSITION))
  #+MCL
  (CL:CHECK-TYPE POSITION CL:FIXNUM)
  (CL:RETURN-FROM NTH-SETTER
   (CL:LET ((SELF (%THE-ARRAY SELF)) (VALUE VALUE) (POSITION POSITION))
    (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF) (CL:TYPE CL:FIXNUM POSITION))
    (CL:SETF (CL:AREF SELF POSITION) VALUE))))

;;; (DEFMETHOD (LAST-SETTER (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFMETHOD LAST-SETTER ((SELF VECTOR) VALUE)
  (CL:RETURN-FROM LAST-SETTER
   (CL:LET
    ((SELF (%THE-ARRAY SELF)) (VALUE VALUE)
     (POSITION (CL:1- (%ARRAY-SIZE SELF))))
    (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF) (CL:TYPE CL:FIXNUM POSITION))
    (CL:SETF (CL:AREF SELF POSITION) VALUE))))

;;; (DEFMETHOD (LENGTH INTEGER) ...)

(CL:DEFMETHOD LENGTH ((SELF VECTOR))
  (CL:RETURN-FROM LENGTH (%ARRAY-SIZE SELF)))

;;; (DEFMETHOD (MEMBER? BOOLEAN) ...)

(CL:DEFMETHOD MEMBER? ((SELF VECTOR) OBJECT)
  (CL:LET* ((ARRAY (%THE-ARRAY SELF)))
   (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR ARRAY))
   (CL:LET*
    ((I NULL-INTEGER) (ITER-000 0) (UPPER-BOUND-000 (CL:1- (LENGTH SELF))))
    (CL:DECLARE (CL:TYPE CL:FIXNUM ITER-000 UPPER-BOUND-000 I))
    (CL:LOOP WHILE (CL:<= ITER-000 UPPER-BOUND-000) DO
     (CL:PROGN (CL:SETQ I ITER-000) (CL:SETQ ITER-000 (CL:1+ ITER-000)))
     (CL:WHEN (EQL? (CL:AREF ARRAY I) OBJECT) (CL:RETURN-FROM MEMBER? TRUE))))
   (CL:RETURN-FROM MEMBER? FALSE)))

;;; (DEFMETHOD (POSITION INTEGER) ...)

(CL:DEFMETHOD POSITION ((SELF VECTOR) OBJECT START)
  "Return the position of `object' within the vector
`self' (counting from zero); or return `null' if `object' does not occur within 
`self' (uses an `eql?' test).  If `start' was supplied as non-`null', only 
consider the portion starting at `start', however, the returned position 
will always be relative to the entire vector."
  (CL:DECLARE (CL:TYPE CL:FIXNUM START))
  #+MCL
  (CL:CHECK-TYPE START CL:FIXNUM)
  (CL:WHEN (NULL? START) (CL:SETQ START 0))
  (CL:LET*
   ((I NULL-INTEGER) (ITER-000 START) (UPPER-BOUND-000 (CL:1- (LENGTH SELF))))
   (CL:DECLARE (CL:TYPE CL:FIXNUM ITER-000 UPPER-BOUND-000 I))
   (CL:LOOP WHILE (CL:<= ITER-000 UPPER-BOUND-000) DO
    (CL:PROGN (CL:SETQ I ITER-000) (CL:SETQ ITER-000 (CL:1+ ITER-000)))
    (CL:WHEN
     (EQL? (CL:AREF (CL:THE CL:SIMPLE-VECTOR (%THE-ARRAY SELF)) I) OBJECT)
     (CL:RETURN-FROM POSITION I))))
  (CL:RETURN-FROM POSITION NULL-INTEGER))

;;; (DEFMETHOD INSERT-AT ...)

(CL:DEFMETHOD INSERT-AT ((SELF VECTOR) OFFSET VALUE)
  (CL:DECLARE (CL:TYPE CL:FIXNUM OFFSET))
  #+MCL
  (CL:CHECK-TYPE OFFSET CL:FIXNUM)
  (CL:WHEN (CL:>= OFFSET (%ARRAY-SIZE SELF))
   (CL:ERROR (NEW-STELLA-EXCEPTION "Out-of-bounds array access")))
  (CL:LET ((SELF (%THE-ARRAY SELF)) (VALUE VALUE) (POSITION OFFSET))
   (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF) (CL:TYPE CL:FIXNUM POSITION))
   (CL:SETF (CL:AREF SELF POSITION) VALUE))
  :VOID)

;;; (DEFMETHOD (COPY (VECTOR OF (LIKE (ANY-VALUE SELF)))) ...)

(CL:DEFMETHOD COPY ((SELF VECTOR))
  "Return a copy of the vector `self'."
  (CL:LET*
   ((SIZE (%ARRAY-SIZE SELF)) (SOURCEARRAY (%THE-ARRAY SELF))
    (COPY (NEW-VECTOR SIZE)) (COPYARRAY (%THE-ARRAY COPY)))
   (CL:DECLARE (CL:TYPE CL:FIXNUM SIZE)
    (CL:TYPE CL:SIMPLE-VECTOR SOURCEARRAY COPYARRAY))
   (CL:LET* ((I NULL-INTEGER) (ITER-000 0) (UPPER-BOUND-000 (CL:1- SIZE)))
    (CL:DECLARE (CL:TYPE CL:FIXNUM ITER-000 UPPER-BOUND-000 I))
    (CL:LOOP WHILE (CL:<= ITER-000 UPPER-BOUND-000) DO
     (CL:PROGN (CL:SETQ I ITER-000) (CL:SETQ ITER-000 (CL:1+ ITER-000)))
     (CL:LET ((SELF COPYARRAY) (VALUE (CL:AREF SOURCEARRAY I)) (POSITION I))
      (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF)
       (CL:TYPE CL:FIXNUM POSITION))
      (CL:SETF (CL:AREF SELF POSITION) VALUE))))
   (CL:RETURN-FROM COPY COPY)))

;;; (DEFMETHOD CLEAR ...)

(CL:DEFMETHOD CLEAR ((SELF VECTOR))
  (CL:LET* ((SIZE (%ARRAY-SIZE SELF)) (ARRAY (%THE-ARRAY SELF)))
   (CL:DECLARE (CL:TYPE CL:FIXNUM SIZE) (CL:TYPE CL:SIMPLE-VECTOR ARRAY))
   (CL:LET* ((I NULL-INTEGER) (ITER-000 0) (UPPER-BOUND-000 (CL:1- SIZE)))
    (CL:DECLARE (CL:TYPE CL:FIXNUM ITER-000 UPPER-BOUND-000 I))
    (CL:LOOP WHILE (CL:<= ITER-000 UPPER-BOUND-000) DO
     (CL:PROGN (CL:SETQ I ITER-000) (CL:SETQ ITER-000 (CL:1+ ITER-000)))
     (CL:LET ((SELF ARRAY) (VALUE NULL) (POSITION I))
      (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF)
       (CL:TYPE CL:FIXNUM POSITION))
      (CL:SETF (CL:AREF SELF POSITION) VALUE)))))
  :VOID)

;;; (DEFUN (VECTOR VECTOR) ...)

(CL:DEFUN VECTOR (CL:&REST VALUES)
  "Return a vector containing `values', in order."
  (CL:LET* ((VECTOR (NEW-VECTOR (CL:LENGTH VALUES))))
   (CL:LET* ((I NULL-INTEGER) (V NULL) (ITER-000 VALUES) (ITER-001 0))
    (CL:DECLARE (CL:TYPE CL:FIXNUM ITER-001 I))
    (CL:LOOP WHILE ITER-000 DO (CL:SETQ V (CL:POP ITER-000))
     (CL:PROGN (CL:SETQ I ITER-001) (CL:SETQ ITER-001 (CL:1+ ITER-001)))
     (CL:LET ((SELF (%THE-ARRAY VECTOR)) (VALUE V) (POSITION I))
      (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF)
       (CL:TYPE CL:FIXNUM POSITION))
      (CL:SETF (CL:AREF SELF POSITION) VALUE))))
   (CL:RETURN-FROM VECTOR VECTOR)))

;;; (DEFMETHOD (OBJECT-EQUAL? BOOLEAN) ...)

(CL:DEFMETHOD OBJECT-EQUAL? ((X VECTOR) Y)
  "Return TRUE iff the vectors `x' and `y' are structurally
equivalent.  Uses `equal?' to test equality of elements."
  (CL:COND
   ((SUBTYPE-OF? (SAFE-PRIMARY-TYPE Y) SGT-COLLECTIONS-STELLA-VECTOR)
    (CL:PROGN
     (CL:WHEN (CL:= (%ARRAY-SIZE X) (%ARRAY-SIZE Y))
      (CL:LET*
       ((ELTY NULL) (ELTX NULL) (VECTOR-000 X) (INDEX-000 0)
        (LENGTH-000 (LENGTH VECTOR-000)) (VECTOR-001 Y) (INDEX-001 0)
        (LENGTH-001 (LENGTH VECTOR-001)))
       (CL:DECLARE
        (CL:TYPE CL:FIXNUM INDEX-000 LENGTH-000 INDEX-001 LENGTH-001))
       (CL:LOOP WHILE
        (CL:AND (CL:< INDEX-000 LENGTH-000) (CL:< INDEX-001 LENGTH-001)) DO
        (CL:PROGN
         (CL:SETQ ELTX
          (CL:AREF (CL:THE CL:SIMPLE-VECTOR (%THE-ARRAY VECTOR-000))
           INDEX-000))
         (CL:SETQ INDEX-000 (CL:1+ INDEX-000)))
        (CL:PROGN
         (CL:SETQ ELTY
          (CL:AREF (CL:THE CL:SIMPLE-VECTOR (%THE-ARRAY VECTOR-001))
           INDEX-001))
         (CL:SETQ INDEX-001 (CL:1+ INDEX-001)))
        (CL:WHEN (CL:NOT (EQUAL? ELTX ELTY))
         (CL:RETURN-FROM OBJECT-EQUAL? FALSE))))
      (CL:RETURN-FROM OBJECT-EQUAL? TRUE))))
   (CL:T))
  (CL:RETURN-FROM OBJECT-EQUAL? FALSE))

;;; (DEFMETHOD INSERT-AT ...)

(CL:DEFMETHOD INSERT-AT ((SELF EXTENSIBLE-VECTOR) OFFSET VALUE)
  (CL:DECLARE (CL:TYPE CL:FIXNUM OFFSET))
  #+MCL
  (CL:CHECK-TYPE OFFSET CL:FIXNUM)
  (CL:WHEN (CL:>= OFFSET (%ARRAY-SIZE SELF))
   (CL:LET* ((SIZE (%ARRAY-SIZE SELF))) (CL:DECLARE (CL:TYPE CL:FIXNUM SIZE))
    (CL:LOOP WHILE (CL:>= OFFSET SIZE) DO (CL:SETQ SIZE (CL:* 2 SIZE)))
    (RESIZE-VECTOR SELF SIZE)))
  (CL:LET ((SELF (%THE-ARRAY SELF)) (VALUE VALUE) (POSITION OFFSET))
   (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF) (CL:TYPE CL:FIXNUM POSITION))
   (CL:SETF (CL:AREF SELF POSITION) VALUE))
  :VOID)

;;; (DEFMETHOD INSERT-AT ...)

(CL:DEFMETHOD INSERT-AT ((SELF EXTENSIBLE-SYMBOL-ARRAY) OFFSET VALUE)
  (CL:DECLARE (CL:TYPE CL:FIXNUM OFFSET))
  #+MCL
  (CL:CHECK-TYPE OFFSET CL:FIXNUM)
  (CL:WHEN (CL:>= OFFSET (%ARRAY-SIZE SELF))
   (CL:LET* ((SIZE (%ARRAY-SIZE SELF))) (CL:DECLARE (CL:TYPE CL:FIXNUM SIZE))
    (CL:LOOP WHILE (CL:>= OFFSET SIZE) DO (CL:SETQ SIZE (CL:* 2 SIZE)))
    (RESIZE-VECTOR SELF SIZE)))
  (CL:LET ((SELF (%THE-ARRAY SELF)) (VALUE VALUE) (POSITION OFFSET))
   (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF) (CL:TYPE CL:FIXNUM POSITION))
   (CL:SETF (CL:AREF SELF POSITION) VALUE))
  (CL:WHEN (CL:> OFFSET (%TOP-SYMBOL-OFFSET SELF))
   (CL:SETF (%TOP-SYMBOL-OFFSET SELF) OFFSET))
  :VOID)

;;; (DEFMETHOD INSERT ...)

(CL:DEFMETHOD INSERT ((SELF VECTOR-SEQUENCE) VALUE)
  "Append `value' to the END of the sequence `self'.  
Resize the array if necessary."
  (CL:LET* ((OLDLENGTH (%SEQUENCE-LENGTH SELF)))
   (CL:DECLARE (CL:TYPE CL:FIXNUM OLDLENGTH))
   (CL:WHEN (CL:= OLDLENGTH (%ARRAY-SIZE SELF))
    (RESIZE-VECTOR SELF (CL:+ (%ARRAY-SIZE SELF) (RESIZE-INCREMENT SELF))))
   (CL:LET ((SELF (%THE-ARRAY SELF)) (VALUE VALUE) (POSITION OLDLENGTH))
    (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF) (CL:TYPE CL:FIXNUM POSITION))
    (CL:SETF (CL:AREF SELF POSITION) VALUE))
   (CL:SETF (%SEQUENCE-LENGTH SELF) (CL:1+ OLDLENGTH)))
  :VOID)

;;; (DEFMETHOD (REMOVE VECTOR-SEQUENCE) ...)

(CL:DEFMETHOD REMOVE ((SELF VECTOR-SEQUENCE) VALUE)
  "Remove `value' from the sequence `self', and left shift
the values after it to close the gap."
  (CL:LET*
   ((ARRAY (%THE-ARRAY SELF)) (FIRSTSHIFTOFFSET -1)
    (LASTSHIFTOFFSET (CL:1- (%SEQUENCE-LENGTH SELF))))
   (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR ARRAY)
    (CL:TYPE CL:FIXNUM FIRSTSHIFTOFFSET LASTSHIFTOFFSET))
   (CL:LET*
    ((I NULL-INTEGER) (ITER-000 0) (UPPER-BOUND-000 LASTSHIFTOFFSET)
     (UNBOUNDED?-000 (NULL? UPPER-BOUND-000)))
    (CL:DECLARE (CL:TYPE CL:FIXNUM ITER-000 UPPER-BOUND-000 I))
    (CL:LOOP WHILE (CL:OR UNBOUNDED?-000 (CL:<= ITER-000 UPPER-BOUND-000)) DO
     (CL:PROGN (CL:SETQ I ITER-000) (CL:SETQ ITER-000 (CL:1+ ITER-000)))
     (CL:WHEN (EQL? (CL:AREF ARRAY I) VALUE)
      (CL:SETQ FIRSTSHIFTOFFSET (CL:1+ I)) (CL:RETURN))))
   (CL:WHEN (CL:= FIRSTSHIFTOFFSET -1) (CL:RETURN-FROM REMOVE SELF))
   (CL:WHEN (CL:<= FIRSTSHIFTOFFSET LASTSHIFTOFFSET)
    (CL:LET*
     ((J NULL-INTEGER) (ITER-001 FIRSTSHIFTOFFSET)
      (UPPER-BOUND-001 LASTSHIFTOFFSET)
      (UNBOUNDED?-001 (NULL? UPPER-BOUND-001)))
     (CL:DECLARE (CL:TYPE CL:FIXNUM ITER-001 UPPER-BOUND-001 J))
     (CL:LOOP WHILE (CL:OR UNBOUNDED?-001 (CL:<= ITER-001 UPPER-BOUND-001)) DO
      (CL:PROGN (CL:SETQ J ITER-001) (CL:SETQ ITER-001 (CL:1+ ITER-001)))
      (CL:LET ((SELF ARRAY) (VALUE (CL:AREF ARRAY J)) (POSITION (CL:1- J)))
       (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF)
        (CL:TYPE CL:FIXNUM POSITION))
       (CL:SETF (CL:AREF SELF POSITION) VALUE)))))
   (CL:LET ((SELF ARRAY) (VALUE NULL) (POSITION LASTSHIFTOFFSET))
    (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR SELF) (CL:TYPE CL:FIXNUM POSITION))
    (CL:SETF (CL:AREF SELF POSITION) VALUE))
   (CL:SETF (%SEQUENCE-LENGTH SELF) LASTSHIFTOFFSET)
   (CL:RETURN-FROM REMOVE SELF)))

;;; (DEFMETHOD (LENGTH INTEGER) ...)

(CL:DEFMETHOD LENGTH ((SELF VECTOR-SEQUENCE))
  (CL:RETURN-FROM LENGTH (%SEQUENCE-LENGTH SELF)))

;;; (DEFMETHOD (ALLOCATE-ITERATOR (ITERATOR OF (LIKE (ANY-VALUE SELF)))) ...)

(CL:DEFMETHOD ALLOCATE-ITERATOR ((SELF VECTOR))
  (CL:LET* ((ITERATOR (NEW-ALL-PURPOSE-ITERATOR)))
   (CL:SETF (%ITERATOR-OBJECT ITERATOR) SELF)
   (CL:SETF (%ITERATOR-INTEGER ITERATOR) 0)
   (CL:SETF (%ITERATOR-SECOND-INTEGER ITERATOR) (%ARRAY-SIZE SELF))
   (CL:SETF (%ITERATOR-NEXT-CODE ITERATOR) (CL:FUNCTION VECTOR-NEXT?))
   (CL:RETURN-FROM ALLOCATE-ITERATOR ITERATOR)))

;;; (DEFUN (VECTOR-NEXT? BOOLEAN) ...)

(CL:DEFUN VECTOR-NEXT? (SELF)
  (CL:LET* ((CURSOR (%ITERATOR-INTEGER SELF)))
   (CL:DECLARE (CL:TYPE CL:FIXNUM CURSOR))
   (CL:WHEN (CL:< CURSOR (%ITERATOR-SECOND-INTEGER SELF))
    (CL:SETF (%VALUE SELF)
     (CL:AREF (CL:THE CL:SIMPLE-VECTOR (%THE-ARRAY (%ITERATOR-OBJECT SELF)))
      CURSOR))
    (CL:SETF (%ITERATOR-INTEGER SELF) (CL:1+ CURSOR))
    (CL:RETURN-FROM VECTOR-NEXT? TRUE))
   (CL:RETURN-FROM VECTOR-NEXT? FALSE)))

;;; (DEFMETHOD (BUT-LAST (ITERATOR OF (LIKE (ANY-VALUE SELF)))) ...)

(CL:DEFMETHOD BUT-LAST ((SELF VECTOR))
  "Generate all but the last element of the vector `self'."
  (CL:LET* ((ITERATOR (NEW-ALL-PURPOSE-ITERATOR)))
   (CL:SETF (%ITERATOR-OBJECT ITERATOR) SELF)
   (CL:SETF (%ITERATOR-INTEGER ITERATOR) 0)
   (CL:SETF (%ITERATOR-SECOND-INTEGER ITERATOR) (CL:1- (%ARRAY-SIZE SELF)))
   (CL:SETF (%ITERATOR-NEXT-CODE ITERATOR) (CL:FUNCTION VECTOR-NEXT?))
   (CL:RETURN-FROM BUT-LAST ITERATOR)))

;;; (DEFGLOBAL *USE-STELLA-HASH-TABLES?* ...)

(CL:DEFVAR *USE-STELLA-HASH-TABLES?* FALSE)

;;; (DEFCLASS STELLA-HASH-TABLE ...)

(CL:DEFCLASS STELLA-HASH-TABLE (ABSTRACT-HASH-TABLE)
  ((THE-TABLE :ALLOCATION :INSTANCE :ACCESSOR %THE-TABLE)
   (SIZE :TYPE CL:FIXNUM :INITFORM NULL-INTEGER :ALLOCATION :INSTANCE
    :ACCESSOR %SIZE)
   (INITIAL-SIZE :TYPE CL:FIXNUM :INITFORM NULL-INTEGER :ALLOCATION :INSTANCE
    :ACCESSOR %INITIAL-SIZE)
   (FREE-ELEMENTS :TYPE CL:FIXNUM :INITFORM NULL-INTEGER :ALLOCATION :INSTANCE
    :ACCESSOR %FREE-ELEMENTS)))

(CL:DEFUN NEW-STELLA-HASH-TABLE ()
  (CL:LET* ((SELF NULL))
   (CL:SETQ SELF (CL:MAKE-INSTANCE (CL:QUOTE STELLA-HASH-TABLE)))
   (CL:SETF (%THE-STELLA-HASH-TABLE SELF) NULL)
   (CL:SETF (%THE-HASH-TABLE SELF) NULL)
   (CL:SETF (%FREE-ELEMENTS SELF) NULL-INTEGER)
   (CL:SETF (%INITIAL-SIZE SELF) 50) (CL:SETF (%SIZE SELF) NULL-INTEGER)
   (CL:SETF (%THE-TABLE SELF) NULL) (INITIALIZE-HASH-TABLE SELF)
   (CL:RETURN-FROM NEW-STELLA-HASH-TABLE SELF)))

(CL:DEFMETHOD PRIMARY-TYPE ((SELF STELLA-HASH-TABLE))
  (CL:RETURN-FROM PRIMARY-TYPE SGT-COLLECTIONS-STELLA-STELLA-HASH-TABLE))

(CL:DEFUN ACCESS-STELLA-HASH-TABLE-SLOT-VALUE (SELF SLOTNAME VALUE SETVALUE?)
  (CL:COND
   ((CL:EQ SLOTNAME SYM-COLLECTIONS-STELLA-SIZE)
    (CL:IF SETVALUE? (CL:SETF (%SIZE SELF) (%WRAPPER-VALUE VALUE))
     (CL:SETQ VALUE (WRAP-INTEGER (%SIZE SELF)))))
   ((CL:EQ SLOTNAME SYM-COLLECTIONS-STELLA-INITIAL-SIZE)
    (CL:IF SETVALUE? (CL:SETF (%INITIAL-SIZE SELF) (%WRAPPER-VALUE VALUE))
     (CL:SETQ VALUE (WRAP-INTEGER (%INITIAL-SIZE SELF)))))
   ((CL:EQ SLOTNAME SYM-COLLECTIONS-STELLA-FREE-ELEMENTS)
    (CL:IF SETVALUE? (CL:SETF (%FREE-ELEMENTS SELF) (%WRAPPER-VALUE VALUE))
     (CL:SETQ VALUE (WRAP-INTEGER (%FREE-ELEMENTS SELF)))))
   (CL:T
    (CL:LET* ((STREAM-000 (NEW-OUTPUT-STRING-STREAM)))
     (%%PRINT-STREAM (%NATIVE-STREAM STREAM-000) "`" SLOTNAME
      "' is not a valid case option")
     (CL:ERROR (NEW-STELLA-EXCEPTION (THE-STRING-READER STREAM-000))))))
  (CL:RETURN-FROM ACCESS-STELLA-HASH-TABLE-SLOT-VALUE VALUE))

;;; (DEFGLOBAL *STELLA-HASH-TABLE-AVG-BUCKET-LENGTH* ...)

(CL:DEFVAR *STELLA-HASH-TABLE-AVG-BUCKET-LENGTH* 1.0d0)
(CL:DECLAIM (CL:TYPE CL:DOUBLE-FLOAT *STELLA-HASH-TABLE-AVG-BUCKET-LENGTH*))

;;; (DEFUN INITIALIZE-STELLA-HASH-TABLE ...)

(CL:DEFUN INITIALIZE-STELLA-HASH-TABLE (SELF)
  (CL:LET*
   ((SIZE
     (PICK-HASH-TABLE-SIZE-PRIME
      (FLOOR
       (CL:/ (%INITIAL-SIZE SELF) *STELLA-HASH-TABLE-AVG-BUCKET-LENGTH*))))
    (TABLE (CL:MAKE-ARRAY SIZE)))
   (CL:DECLARE (CL:TYPE CL:FIXNUM SIZE) (CL:TYPE CL:SIMPLE-VECTOR TABLE))
   (CL:LET* ((I NULL-INTEGER) (ITER-000 0) (UPPER-BOUND-000 (CL:1- SIZE)))
    (CL:DECLARE (CL:TYPE CL:FIXNUM ITER-000 UPPER-BOUND-000 I))
    (CL:LOOP WHILE (CL:<= ITER-000 UPPER-BOUND-000) DO
     (CL:PROGN (CL:SETQ I ITER-000) (CL:SETQ ITER-000 (CL:1+ ITER-000)))
     (CL:SETF (CL:AREF TABLE I) NULL)))
   (CL:SETF (%THE-TABLE SELF) TABLE) (CL:SETF (%SIZE SELF) SIZE)
   (CL:SETF (%FREE-ELEMENTS SELF)
    (FLOOR (CL:* SIZE *STELLA-HASH-TABLE-AVG-BUCKET-LENGTH*))))
  :VOID)

;;; (DEFUN (MAKE-STELLA-HASH-TABLE STELLA-HASH-TABLE) ...)

(CL:DEFUN MAKE-STELLA-HASH-TABLE (INITIALSIZE)
  (CL:DECLARE (CL:TYPE CL:FIXNUM INITIALSIZE))
  #+MCL
  (CL:CHECK-TYPE INITIALSIZE CL:FIXNUM)
  (CL:LET* ((TABLE (CL:MAKE-INSTANCE (CL:QUOTE STELLA-HASH-TABLE))))
   (CL:SETF (%THE-TABLE TABLE) NULL) (CL:SETF (%SIZE TABLE) NULL-INTEGER)
   (CL:SETF (%INITIAL-SIZE TABLE) INITIALSIZE)
   (CL:SETF (%FREE-ELEMENTS TABLE) NULL-INTEGER)
   (CL:RETURN-FROM MAKE-STELLA-HASH-TABLE TABLE)))

;;; (DEFUN REHASH-STELLA-HASH-TABLE ...)

(CL:DEFUN REHASH-STELLA-HASH-TABLE (SELF NEWSIZE)
  (CL:DECLARE (CL:TYPE CL:FIXNUM NEWSIZE))
  #+MCL
  (CL:CHECK-TYPE NEWSIZE CL:FIXNUM)
  (CL:WHEN (CL:EQ (%THE-TABLE SELF) NULL) (INITIALIZE-STELLA-HASH-TABLE SELF)
   (CL:RETURN-FROM REHASH-STELLA-HASH-TABLE))
  (CL:LET*
   ((SIZE (%SIZE SELF)) (TABLE (%THE-TABLE SELF))
    (NEWTABLE (CL:MAKE-ARRAY NEWSIZE)) (NEWBUCKETINDEX 0) (NEWBUCKET NULL)
    (CURSOR NULL) (CURRENT NULL))
   (CL:DECLARE (CL:TYPE CL:FIXNUM SIZE NEWBUCKETINDEX)
    (CL:TYPE CL:SIMPLE-VECTOR TABLE NEWTABLE))
   (CL:LET* ((I NULL-INTEGER) (ITER-000 0) (UPPER-BOUND-000 (CL:1- NEWSIZE)))
    (CL:DECLARE (CL:TYPE CL:FIXNUM ITER-000 UPPER-BOUND-000 I))
    (CL:LOOP WHILE (CL:<= ITER-000 UPPER-BOUND-000) DO
     (CL:PROGN (CL:SETQ I ITER-000) (CL:SETQ ITER-000 (CL:1+ ITER-000)))
     (CL:SETF (CL:AREF NEWTABLE I) NULL)))
   (CL:LET* ((I NULL-INTEGER) (ITER-001 0) (UPPER-BOUND-001 (CL:1- SIZE)))
    (CL:DECLARE (CL:TYPE CL:FIXNUM ITER-001 UPPER-BOUND-001 I))
    (CL:LOOP WHILE (CL:<= ITER-001 UPPER-BOUND-001) DO
     (CL:PROGN (CL:SETQ I ITER-001) (CL:SETQ ITER-001 (CL:1+ ITER-001)))
     (CL:SETQ CURSOR (CL:AREF TABLE I))
     (CL:LOOP WHILE (CL:NOT (CL:EQ CURSOR NULL)) DO
      (CL:SETQ NEWBUCKETINDEX
       (CL:THE CL:FIXNUM
        (CL:MOD (CL:THE CL:FIXNUM (HASH-CODE (%KEY CURSOR))) NEWSIZE)))
      (CL:SETQ NEWBUCKET (CL:AREF NEWTABLE NEWBUCKETINDEX))
      (CL:SETQ CURRENT CURSOR) (CL:SETQ CURSOR (%REST CURSOR))
      (CL:COND
       ((CL:NOT (CL:EQ NEWBUCKET NULL))
        (CL:SETF (%REST CURRENT) (%REST NEWBUCKET))
        (CL:SETF (%REST NEWBUCKET) CURRENT))
       (CL:T (CL:SETF (CL:AREF NEWTABLE NEWBUCKETINDEX) CURRENT)
        (CL:SETF (%REST CURRENT) NULL))))))
   (CL:SETF (%THE-TABLE SELF) NEWTABLE) (CL:SETF (%SIZE SELF) NEWSIZE)
   (CL:SETF (%FREE-ELEMENTS SELF)
    (MAX
     (CL:+ (%FREE-ELEMENTS SELF)
      (FLOOR
       (CL:* (CL:- NEWSIZE SIZE) *STELLA-HASH-TABLE-AVG-BUCKET-LENGTH*)))
     0)))
  :VOID)

;;; (DEFUN STELLA-HASH-TABLE-INSERT-AT ...)

(CL:DEFUN STELLA-HASH-TABLE-INSERT-AT (SELF KEY VALUE)
  (CL:LET*
   ((TABLE (%THE-TABLE SELF)) (FREE (%FREE-ELEMENTS SELF)) (BUCKETINDEX 0)
    (BUCKET NULL))
   (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR TABLE)
    (CL:TYPE CL:FIXNUM FREE BUCKETINDEX))
   (CL:WHEN (CL:EQ TABLE NULL) (INITIALIZE-STELLA-HASH-TABLE SELF)
    (CL:SETQ TABLE (%THE-TABLE SELF)) (CL:SETQ FREE (%FREE-ELEMENTS SELF)))
   (CL:WHEN (CL:= FREE 0)
    (REHASH-STELLA-HASH-TABLE SELF
     (PICK-HASH-TABLE-SIZE-PRIME (CL:1+ (%SIZE SELF))))
    (CL:SETQ TABLE (%THE-TABLE SELF)) (CL:SETQ FREE (%FREE-ELEMENTS SELF)))
   (CL:SETQ BUCKETINDEX
    (CL:THE CL:FIXNUM
     (CL:MOD (CL:THE CL:FIXNUM (HASH-CODE KEY))
      (CL:THE CL:FIXNUM (%SIZE SELF)))))
   (CL:SETQ BUCKET (CL:AREF TABLE BUCKETINDEX))
   (CL:COND
    ((CL:EQ BUCKET NULL)
     (CL:LET* ((SELF-000 (NEW-KV-CONS))) (CL:SETF (%KEY SELF-000) KEY)
      (CL:SETF (%VALUE SELF-000) VALUE)
      (CL:SETF (CL:AREF TABLE BUCKETINDEX) SELF-000))
     (CL:SETF (%FREE-ELEMENTS SELF) (CL:1- FREE)))
    (CL:T
     (CL:LET* ((CURSOR BUCKET))
      (CL:LOOP WHILE
       (CL:AND (CL:NOT (CL:EQ CURSOR NULL)) (CL:NOT (EQL? (%KEY CURSOR) KEY)))
       DO (CL:SETQ CURSOR (%REST CURSOR)))
      (CL:COND ((CL:NOT (CL:EQ CURSOR NULL)) (CL:SETF (%VALUE CURSOR) VALUE))
       (CL:T
        (CL:LET* ((SELF-001 (NEW-KV-CONS))) (CL:SETF (%KEY SELF-001) KEY)
         (CL:SETF (%VALUE SELF-001) VALUE)
         (CL:SETF (%REST SELF-001) (%REST BUCKET))
         (CL:SETF (%REST BUCKET) SELF-001))
        (CL:SETF (%FREE-ELEMENTS SELF) (CL:1- FREE))))))))
  :VOID)

;;; (DEFUN STELLA-HASH-TABLE-REMOVE-AT ...)

(CL:DEFUN STELLA-HASH-TABLE-REMOVE-AT (SELF KEY)
  (CL:LET* ((TABLE (%THE-TABLE SELF)) (BUCKETINDEX 0) (BUCKET NULL))
   (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR TABLE)
    (CL:TYPE CL:FIXNUM BUCKETINDEX))
   (CL:WHEN (CL:EQ TABLE NULL) (CL:RETURN-FROM STELLA-HASH-TABLE-REMOVE-AT))
   (CL:SETQ BUCKETINDEX
    (CL:THE CL:FIXNUM
     (CL:MOD (CL:THE CL:FIXNUM (HASH-CODE KEY))
      (CL:THE CL:FIXNUM (%SIZE SELF)))))
   (CL:SETQ BUCKET (CL:AREF TABLE BUCKETINDEX))
   (CL:COND ((CL:EQ BUCKET NULL) (CL:RETURN-FROM STELLA-HASH-TABLE-REMOVE-AT))
    ((EQL? (%KEY BUCKET) KEY)
     (CL:SETF (CL:AREF TABLE BUCKETINDEX) (%REST BUCKET))
     (CL:SETF (%FREE-ELEMENTS SELF) (CL:1+ (%FREE-ELEMENTS SELF)))
     (CL:RETURN-FROM STELLA-HASH-TABLE-REMOVE-AT))
    (CL:T
     (CL:LET* ((TRAILER BUCKET)) (CL:SETQ BUCKET (%REST BUCKET))
      (CL:LOOP WHILE (CL:NOT (CL:EQ BUCKET NULL)) DO
       (CL:COND
        ((EQL? (%KEY BUCKET) KEY) (CL:SETF (%REST TRAILER) (%REST BUCKET))
         (CL:SETF (%FREE-ELEMENTS SELF) (CL:1+ (%FREE-ELEMENTS SELF)))
         (CL:RETURN-FROM STELLA-HASH-TABLE-REMOVE-AT))
        (CL:T (CL:SETQ TRAILER BUCKET) (CL:SETQ BUCKET (%REST BUCKET)))))))))
  :VOID)

;;; (DEFUN (STELLA-HASH-TABLE-LOOKUP (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFUN STELLA-HASH-TABLE-LOOKUP (SELF KEY)
  (CL:LET* ((TABLE (%THE-TABLE SELF)) (BUCKET NULL))
   (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR TABLE))
   (CL:WHEN (CL:EQ TABLE NULL) (CL:RETURN-FROM STELLA-HASH-TABLE-LOOKUP NULL))
   (CL:SETQ BUCKET
    (CL:AREF TABLE
     (CL:THE CL:FIXNUM
      (CL:MOD (CL:THE CL:FIXNUM (HASH-CODE KEY))
       (CL:THE CL:FIXNUM (%SIZE SELF))))))
   (CL:LOOP WHILE (CL:NOT (CL:EQ BUCKET NULL)) DO
    (CL:IF (EQL? (%KEY BUCKET) KEY)
     (CL:RETURN-FROM STELLA-HASH-TABLE-LOOKUP (%VALUE BUCKET))
     (CL:SETQ BUCKET (%REST BUCKET))))
   (CL:RETURN-FROM STELLA-HASH-TABLE-LOOKUP NULL)))

;;; (DEFUN (STELLA-STRING-HASH-TABLE-LOOKUP (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFUN STELLA-STRING-HASH-TABLE-LOOKUP (SELF KEY)
  (CL:DECLARE (CL:TYPE CL:SIMPLE-STRING KEY))
  #+MCL
  (CL:CHECK-TYPE KEY CL:SIMPLE-STRING)
  (CL:LET* ((TABLE (%THE-TABLE SELF)) (BUCKET NULL))
   (CL:DECLARE (CL:TYPE CL:SIMPLE-VECTOR TABLE))
   (CL:WHEN (CL:EQ TABLE NULL)
    (CL:RETURN-FROM STELLA-STRING-HASH-TABLE-LOOKUP NULL))
   (CL:SETQ BUCKET
    (CL:AREF TABLE
     (CL:THE CL:FIXNUM
      (CL:MOD (CL:THE CL:FIXNUM (CL:SXHASH KEY))
       (CL:THE CL:FIXNUM (%SIZE SELF))))))
   (CL:LOOP WHILE (CL:NOT (CL:EQ BUCKET NULL)) DO
    (CL:IF (STRING-EQL? (%WRAPPER-VALUE (%KEY BUCKET)) KEY)
     (CL:RETURN-FROM STELLA-STRING-HASH-TABLE-LOOKUP (%VALUE BUCKET))
     (CL:SETQ BUCKET (%REST BUCKET))))
   (CL:RETURN-FROM STELLA-STRING-HASH-TABLE-LOOKUP NULL)))

;;; (DEFMETHOD (LOOKUP (LIKE (ANY-VALUE SELF))) ...)

(CL:DEFMETHOD LOOKUP ((SELF STELLA-HASH-TABLE) KEY)
  (CL:RETURN-FROM LOOKUP (STELLA-HASH-TABLE-LOOKUP SELF KEY)))

;;; (DEFMETHOD INSERT-AT ...)

(CL:DEFMETHOD INSERT-AT ((SELF STELLA-HASH-TABLE) KEY VALUE)
  (STELLA-HASH-TABLE-INSERT-AT SELF KEY VALUE)
  (CL:RETURN-FROM INSERT-AT)
  :VOID)

;;; (DEFMETHOD REMOVE-AT ...)

(CL:DEFMETHOD REMOVE-AT ((SELF STELLA-HASH-TABLE) KEY)
  (STELLA-HASH-TABLE-REMOVE-AT SELF KEY)
  (CL:RETURN-FROM REMOVE-AT)
  :VOID)

(CL:DEFUN HELP-STARTUP-COLLECTIONS1 ()
  (CL:PROGN
   (CL:SETQ SGT-COLLECTIONS-STELLA-COLLECTION
    (INTERN-RIGID-SYMBOL-WRT-MODULE "COLLECTION" NULL 1))
   (CL:SETQ SYM-COLLECTIONS-STELLA-VECTOR
    (INTERN-RIGID-SYMBOL-WRT-MODULE "VECTOR" NULL 0))
   (CL:SETQ KWD-COLLECTIONS-CPP (INTERN-RIGID-SYMBOL-WRT-MODULE "CPP" NULL 2))
   (CL:SETQ KWD-COLLECTIONS-FUNCTION
    (INTERN-RIGID-SYMBOL-WRT-MODULE "FUNCTION" NULL 2))
   (CL:SETQ SGT-COLLECTIONS-STELLA-VECTOR
    (INTERN-RIGID-SYMBOL-WRT-MODULE "VECTOR" NULL 1))
   (CL:SETQ SGT-COLLECTIONS-STELLA-STELLA-HASH-TABLE
    (INTERN-RIGID-SYMBOL-WRT-MODULE "STELLA-HASH-TABLE" NULL 1))
   (CL:SETQ SYM-COLLECTIONS-STELLA-SIZE
    (INTERN-RIGID-SYMBOL-WRT-MODULE "SIZE" NULL 0))
   (CL:SETQ SYM-COLLECTIONS-STELLA-INITIAL-SIZE
    (INTERN-RIGID-SYMBOL-WRT-MODULE "INITIAL-SIZE" NULL 0))
   (CL:SETQ SYM-COLLECTIONS-STELLA-FREE-ELEMENTS
    (INTERN-RIGID-SYMBOL-WRT-MODULE "FREE-ELEMENTS" NULL 0))
   (CL:SETQ SYM-COLLECTIONS-STELLA-STARTUP-COLLECTIONS
    (INTERN-RIGID-SYMBOL-WRT-MODULE "STARTUP-COLLECTIONS" NULL 0))
   (CL:SETQ SYM-COLLECTIONS-STELLA-METHOD-STARTUP-CLASSNAME
    (INTERN-RIGID-SYMBOL-WRT-MODULE "METHOD-STARTUP-CLASSNAME" NULL 0)))
  :VOID)

(CL:DEFUN HELP-STARTUP-COLLECTIONS2 ()
  (CL:PROGN
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (MEMBER? BOOLEAN) ((SELF COLLECTION) (OBJECT OBJECT)) :DOCUMENTATION \"Return true iff `object' is a member of the collection `self'.\" :PUBLIC? TRUE)"
    (CL:FUNCTION MEMBER?) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (REMOVE-DUPLICATES (LIKE SELF)) ((SELF COLLECTION)) :DOCUMENTATION \"Return `self' with duplicates removed.  Preserves the
original order of the remaining members.\" :PUBLIC? TRUE :ABSTRACT? TRUE)"
    NULL NULL)
   (DEFINE-FUNCTION-OBJECT "STELLA-COLLECTION?"
    "(DEFUN (STELLA-COLLECTION? BOOLEAN) ((SELF OBJECT)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if `self' is a native collection.\")"
    (CL:FUNCTION STELLA-COLLECTION?) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (NO-DUPLICATES? BOOLEAN) ((SELF COLLECTION)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if the collection `self' forbids duplicate values.\")"
    (CL:FUNCTION NO-DUPLICATES?) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (NO-DUPLICATES? BOOLEAN) ((SELF SET)) :PUBLIC? TRUE)"
    (CL:FUNCTION NO-DUPLICATES?) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (ORDERED? BOOLEAN) ((SELF COLLECTION)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if the collection `self' is ordered.\")"
    (CL:FUNCTION ORDERED?) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (ORDERED? BOOLEAN) ((SELF SEQUENCE)) :PUBLIC? TRUE)"
    (CL:FUNCTION ORDERED?) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (EMPTY? BOOLEAN) ((SELF SET)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if the set `self' has no members.\")"
    (CL:FUNCTION EMPTY?) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (NON-EMPTY? BOOLEAN) ((SELF SET)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if the set `self' has at least one member.\")"
    (CL:FUNCTION NON-EMPTY?) NULL)
   (DEFINE-FUNCTION-OBJECT "TERMINATE-SET?"
    "(DEFUN (TERMINATE-SET? BOOLEAN) ((SELF SET)))"
    (CL:FUNCTION TERMINATE-SET?) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (MEMBER? BOOLEAN) ((SELF SET) (OBJECT (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Return `true' iff `object' is a member of the set `self'.
Uses an `eql?' test.\" :PUBLIC? TRUE)" (CL:FUNCTION MEMBER?) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (LENGTH INTEGER) ((SELF SET)) :PUBLIC? TRUE)"
    (CL:FUNCTION LENGTH) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD INSERT ((SELF SET) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :DOCUMENTATION \"Add `value' to the set `self'.  First checks for duplicate.\")"
    (CL:FUNCTION INSERT) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (REMOVE SET) ((SELF SET) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :DOCUMENTATION \"Remove all entries in `self' that match `value'.\")"
    (CL:FUNCTION REMOVE) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (REVERSE SET) ((SELF SET)) :PUBLIC? TRUE)"
    (CL:FUNCTION REVERSE) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (COPY (SET OF (LIKE (ANY-VALUE SELF)))) ((SELF SET)) :PUBLIC? TRUE :DOCUMENTATION \"Return a copy of the set `self'.  The conses in the copy are
freshly allocated.\")" (CL:FUNCTION COPY) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD CLEAR ((SELF SET)) :PUBLIC? TRUE :DOCUMENTATION \"Make `self' an empty set.\")"
    (CL:FUNCTION CLEAR) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (ALLOCATE-ITERATOR (SET-ITERATOR OF (LIKE (ANY-VALUE SELF)))) ((SELF SET)))"
    (CL:FUNCTION ALLOCATE-ITERATOR) NULL)
   (DEFINE-FUNCTION-OBJECT "TERMINATE-SET-ITERATOR?"
    "(DEFUN (TERMINATE-SET-ITERATOR? BOOLEAN) ((SELF SET-ITERATOR)))"
    (CL:FUNCTION TERMINATE-SET-ITERATOR?) NULL)
   (DEFINE-METHOD-OBJECT "(DEFMETHOD (NEXT? BOOLEAN) ((SELF SET-ITERATOR)))"
    (CL:FUNCTION NEXT?) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (VALUE-SETTER OBJECT) ((SELF SET-ITERATOR) (VALUE (LIKE (ANY-VALUE SELF)))))"
    (CL:FUNCTION VALUE-SETTER) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD PRINT-VECTOR ((SELF VECTOR) (STREAM NATIVE-OUTPUT-STREAM)))"
    (CL:FUNCTION PRINT-VECTOR) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (EMPTY? BOOLEAN) ((SELF VECTOR)) :DOCUMENTATION \"Return `true' if `self' has length 0.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (EQL? (ARRAY-SIZE SELF) 0)))"
    (CL:FUNCTION EMPTY?) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (NON-EMPTY? BOOLEAN) ((SELF VECTOR)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if `self' has length > 0.\" :GLOBALLY-INLINE? TRUE (RETURN (> (ARRAY-SIZE SELF) 0)))"
    (CL:FUNCTION NON-EMPTY?) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (FIRST (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH SELF 0)))"
    (CL:FUNCTION FIRST) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (SECOND (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH SELF 1)))"
    (CL:FUNCTION SECOND) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (THIRD (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH SELF 2)))"
    (CL:FUNCTION THIRD) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (FOURTH (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH SELF 3)))"
    (CL:FUNCTION FOURTH) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (FIFTH (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH SELF 4)))"
    (CL:FUNCTION FIFTH) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (NTH (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (POSITION INTEGER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH (THE-ARRAY SELF) POSITION)))"
    (CL:FUNCTION NTH) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (LAST (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :DOCUMENTATION \"Return the last item in the vector `self'.\")"
    (CL:FUNCTION LAST) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (FIRST-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH SELF 0) VALUE)))"
    (CL:FUNCTION FIRST-SETTER) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (SECOND-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH SELF 1) VALUE)))"
    (CL:FUNCTION SECOND-SETTER) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (THIRD-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH SELF 2) VALUE)))"
    (CL:FUNCTION THIRD-SETTER) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (FOURTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH SELF 3) VALUE)))"
    (CL:FUNCTION FOURTH-SETTER) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (FIFTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH SELF 4) VALUE)))"
    (CL:FUNCTION FIFTH-SETTER) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (NTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF))) (POSITION INTEGER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH (THE-ARRAY SELF) POSITION) VALUE)))"
    (CL:FUNCTION NTH-SETTER) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (LAST-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)"
    (CL:FUNCTION LAST-SETTER) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (LENGTH INTEGER) ((SELF VECTOR)) :PUBLIC? TRUE)"
    (CL:FUNCTION LENGTH) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (MEMBER? BOOLEAN) ((SELF VECTOR) (OBJECT OBJECT)) :PUBLIC? TRUE)"
    (CL:FUNCTION MEMBER?) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (POSITION INTEGER) ((SELF VECTOR) (OBJECT OBJECT) (START INTEGER)) :DOCUMENTATION \"Return the position of `object' within the vector
`self' (counting from zero); or return `null' if `object' does not occur within 
`self' (uses an `eql?' test).  If `start' was supplied as non-`null', only 
consider the portion starting at `start', however, the returned position 
will always be relative to the entire vector.\" :PUBLIC? TRUE)"
    (CL:FUNCTION POSITION) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD INSERT-AT ((SELF VECTOR) (OFFSET INTEGER) (VALUE (LIKE (ANY-VALUE SELF)))))"
    (CL:FUNCTION INSERT-AT) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (COPY (VECTOR OF (LIKE (ANY-VALUE SELF)))) ((SELF VECTOR)) :DOCUMENTATION \"Return a copy of the vector `self'.\" :PUBLIC? TRUE)"
    (CL:FUNCTION COPY) NULL)
   (DEFINE-METHOD-OBJECT "(DEFMETHOD CLEAR ((SELF VECTOR)) :PUBLIC? TRUE)"
    (CL:FUNCTION CLEAR) NULL)
   (DEFINE-FUNCTION-OBJECT "VECTOR"
    "(DEFUN (VECTOR VECTOR) (|&REST| (VALUES OBJECT)) :DOCUMENTATION \"Return a vector containing `values', in order.\" :PUBLIC? TRUE)"
    (CL:FUNCTION VECTOR) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (OBJECT-EQUAL? BOOLEAN) ((X VECTOR) (Y OBJECT)) :DOCUMENTATION \"Return TRUE iff the vectors `x' and `y' are structurally
equivalent.  Uses `equal?' to test equality of elements.\" :PUBLIC? TRUE)"
    (CL:FUNCTION OBJECT-EQUAL?) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD INSERT-AT ((SELF EXTENSIBLE-VECTOR) (OFFSET INTEGER) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)"
    (CL:FUNCTION INSERT-AT) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD INSERT-AT ((SELF EXTENSIBLE-SYMBOL-ARRAY) (OFFSET INTEGER) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)"
    (CL:FUNCTION INSERT-AT) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD INSERT ((SELF VECTOR-SEQUENCE) (VALUE (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Append `value' to the END of the sequence `self'.  
Resize the array if necessary.\" :PUBLIC? TRUE)" (CL:FUNCTION INSERT) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (REMOVE VECTOR-SEQUENCE) ((SELF VECTOR-SEQUENCE) (VALUE (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Remove `value' from the sequence `self', and left shift
the values after it to close the gap.\" :PUBLIC? TRUE)" (CL:FUNCTION REMOVE)
    NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (LENGTH INTEGER) ((SELF VECTOR-SEQUENCE)) :PUBLIC? TRUE)"
    (CL:FUNCTION LENGTH) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (ALLOCATE-ITERATOR (ITERATOR OF (LIKE (ANY-VALUE SELF)))) ((SELF VECTOR)))"
    (CL:FUNCTION ALLOCATE-ITERATOR) NULL)
   (DEFINE-FUNCTION-OBJECT "VECTOR-NEXT?"
    "(DEFUN (VECTOR-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))"
    (CL:FUNCTION VECTOR-NEXT?) NULL)
   (DEFINE-METHOD-OBJECT
    "(DEFMETHOD (BUT-LAST (ITERATOR OF (LIKE (ANY-VALUE SELF)))) ((SELF VECTOR)) :DOCUMENTATION \"Generate all but the last element of the vector `self'.\" :PUBLIC? TRUE)"
    (CL:FUNCTION BUT-LAST) NULL)
   (DEFINE-FUNCTION-OBJECT "INITIALIZE-STELLA-HASH-TABLE"
    "(DEFUN INITIALIZE-STELLA-HASH-TABLE ((SELF STELLA-HASH-TABLE)))"
    (CL:FUNCTION INITIALIZE-STELLA-HASH-TABLE) NULL)
   (DEFINE-FUNCTION-OBJECT "MAKE-STELLA-HASH-TABLE"
    "(DEFUN (MAKE-STELLA-HASH-TABLE STELLA-HASH-TABLE) ((INITIALSIZE INTEGER)) :CONSTRUCTOR? TRUE)"
    (CL:FUNCTION MAKE-STELLA-HASH-TABLE) NULL)
   (DEFINE-FUNCTION-OBJECT "REHASH-STELLA-HASH-TABLE"
    "(DEFUN REHASH-STELLA-HASH-TABLE ((SELF STELLA-HASH-TABLE) (NEWSIZE INTEGER)))"
    (CL:FUNCTION REHASH-STELLA-HASH-TABLE) NULL)
   (DEFINE-FUNCTION-OBJECT "STELLA-HASH-TABLE-INSERT-AT"
    "(DEFUN STELLA-HASH-TABLE-INSERT-AT ((SELF STELLA-HASH-TABLE) (KEY OBJECT) (VALUE OBJECT)))"
    (CL:FUNCTION STELLA-HASH-TABLE-INSERT-AT) NULL)
   (DEFINE-FUNCTION-OBJECT "STELLA-HASH-TABLE-REMOVE-AT"
    "(DEFUN STELLA-HASH-TABLE-REMOVE-AT ((SELF STELLA-HASH-TABLE) (KEY OBJECT)))"
    (CL:FUNCTION STELLA-HASH-TABLE-REMOVE-AT) NULL)
   (DEFINE-FUNCTION-OBJECT "STELLA-HASH-TABLE-LOOKUP"
    "(DEFUN (STELLA-HASH-TABLE-LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF STELLA-HASH-TABLE) (KEY OBJECT)))"
    (CL:FUNCTION STELLA-HASH-TABLE-LOOKUP) NULL))
  :VOID)

(CL:DEFUN STARTUP-COLLECTIONS ()
  (CL:LET* ((*MODULE* *STELLA-MODULE*) (*CONTEXT* *MODULE*))
   (CL:DECLARE (CL:SPECIAL *MODULE* *CONTEXT*))
   (CL:WHEN (CURRENT-STARTUP-TIME-PHASE? 2) (HELP-STARTUP-COLLECTIONS1))
   (CL:WHEN (CURRENT-STARTUP-TIME-PHASE? 4)
    (CL:SETQ *USE-STELLA-HASH-TABLES?* FALSE))
   (CL:WHEN (CURRENT-STARTUP-TIME-PHASE? 5)
    (CL:LET*
     ((CLASS
       (DEFINE-CLASS-FROM-STRINGIFIED-SOURCE "STELLA-HASH-TABLE"
        "(DEFCLASS STELLA-HASH-TABLE (ABSTRACT-HASH-TABLE) :PARAMETERS ((ANY-KEY :TYPE OBJECT) (ANY-VALUE :TYPE OBJECT)) :SLOTS ((THE-TABLE :TYPE (ARRAY () OF KV-CONS)) (SIZE :TYPE INTEGER) (INITIAL-SIZE :TYPE INTEGER :INITIALLY 50) (FREE-ELEMENTS :TYPE INTEGER)))")))
     (CL:SETF (%CLASS-CONSTRUCTOR-CODE CLASS)
      (CL:FUNCTION NEW-STELLA-HASH-TABLE))
     (CL:SETF (%CLASS-SLOT-ACCESSOR-CODE CLASS)
      (CL:FUNCTION ACCESS-STELLA-HASH-TABLE-SLOT-VALUE))))
   (CL:WHEN (CURRENT-STARTUP-TIME-PHASE? 6) (FINALIZE-CLASSES))
   (CL:WHEN (CURRENT-STARTUP-TIME-PHASE? 7) (HELP-STARTUP-COLLECTIONS2)
    (DEFINE-FUNCTION-OBJECT "STELLA-STRING-HASH-TABLE-LOOKUP"
     "(DEFUN (STELLA-STRING-HASH-TABLE-LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF STELLA-HASH-TABLE) (KEY STRING)))"
     (CL:FUNCTION STELLA-STRING-HASH-TABLE-LOOKUP) NULL)
    (DEFINE-METHOD-OBJECT
     "(DEFMETHOD (LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF STELLA-HASH-TABLE) (KEY (LIKE (ANY-KEY SELF)))) :PUBLIC? TRUE)"
     (CL:FUNCTION LOOKUP) NULL)
    (DEFINE-METHOD-OBJECT
     "(DEFMETHOD INSERT-AT ((SELF STELLA-HASH-TABLE) (KEY (LIKE (ANY-KEY SELF))) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)"
     (CL:FUNCTION INSERT-AT) NULL)
    (DEFINE-METHOD-OBJECT
     "(DEFMETHOD REMOVE-AT ((SELF STELLA-HASH-TABLE) (KEY (LIKE (ANY-KEY SELF)))) :PUBLIC? TRUE)"
     (CL:FUNCTION REMOVE-AT) NULL)
    (DEFINE-FUNCTION-OBJECT "STARTUP-COLLECTIONS"
     "(DEFUN STARTUP-COLLECTIONS () :PUBLIC? TRUE)"
     (CL:FUNCTION STARTUP-COLLECTIONS) NULL)
    (CL:LET*
     ((FUNCTION (LOOKUP-FUNCTION SYM-COLLECTIONS-STELLA-STARTUP-COLLECTIONS)))
     (SET-DYNAMIC-SLOT-VALUE (%DYNAMIC-SLOTS FUNCTION)
      SYM-COLLECTIONS-STELLA-METHOD-STARTUP-CLASSNAME
      (WRAP-STRING "_Startup-Collections") NULL-STRING-WRAPPER)))
   (CL:WHEN (CURRENT-STARTUP-TIME-PHASE? 8) (FINALIZE-SLOTS)
    (CLEANUP-UNFINALIZED-CLASSES))
   (CL:WHEN (CURRENT-STARTUP-TIME-PHASE? 9)
    (REGISTER-NATIVE-NAME SYM-COLLECTIONS-STELLA-VECTOR KWD-COLLECTIONS-CPP
     KWD-COLLECTIONS-FUNCTION)
    (DEFINE-STELLA-GLOBAL-VARIABLE-FROM-STRINGIFIED-SOURCE
     "(DEFGLOBAL *USE-STELLA-HASH-TABLES?* BOOLEAN FALSE)")
    (DEFINE-STELLA-GLOBAL-VARIABLE-FROM-STRINGIFIED-SOURCE
     "(DEFGLOBAL *STELLA-HASH-TABLE-AVG-BUCKET-LENGTH* FLOAT 1.0)")))
  :VOID)
